import argparse
from typing import List

from cython_api import trial_division


parser = argparse.ArgumentParser(description='Large Integer Factorization')
parser.add_argument('integer', type=int, help='An integer to factorize')
parser.add_argument('-a', '--algo', type=str, default='trail_division', choices=['trail_division'],
                    help='Algorithm to perform factorization')


def main():
    args = parser.parse_args()
    algorithm = args.algo
    number = args.integer
    res: List[str] = []
    if algorithm == "trail_division":
        res = trial_division(number)
    else:
        print(f"No such algorithm {algorithm}")
        exit(1)
    print(', '.join(map(str, res)))
    exit(0)


if __name__ == '__main__':
    main()



# distutils: language=c++

from lif cimport cpp_vector
from libcpp.string cimport string
from typing import List


cdef extern from "cpp_code/algorithms_interface.hpp":
    cpp_vector.vector[string] cpp_trial_division(char * n)

def trial_division(number: int) -> List[int]:
    string_param_utf8 = str(number).encode('utf-8')
    cdef cpp_vector.vector[string] v = cpp_trial_division(string_param_utf8)
    ret_list = []
    for i in range(v.size()):
        ret_list.append(int(v.at(i).decode("utf-8")))
    return ret_list



# Tutorial used:
# https://medium.com/@joel.barmettler/how-to-upload-your-python-package-to-pypi-65edc5fe9c56
from setuptools import setup, Extension


ext_modules = [
    Extension('cython_api', sources=['lif/cython_api.pyx',
                                     'lif/cpp_code/bigint.cpp',
                                     'lif/cpp_code/fft.cpp',
                                     'lif/cpp_code/algorithms_interface.cpp',
                                     'lif/cpp_code/algorithms/trail_division_algorithm.cpp'], language='c++'),
]  # TODO: Turn that to glob("**/*.cpp")

for e_m in ext_modules:
    e_m.compiler_directives = {"language_level": "3"}

setup(
    name='lif',
    packages=['lif'],
    version='0.1.7',
    license='MIT',
    description='Lif lib',
    author='Yonlif & Eilonlif',
    author_email='yonlif@gmail.com',
    url='https://github.com/yonlif/lif',
    download_url='https://github.com/yonlif/lif/archive/refs/tags/0.1.7.tar.gz',
    keywords=['Large', 'Integer', 'Factorization', 'Lif'],
    scripts=["./scripts/lif"],
    install_requires=[],
    ext_modules=ext_modules,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
)
